# coding=utf-8
"""Import OpenStudio SDK classes in different Python environments."""
import sys
import os
from honeybee_energy.config import folders as hbe_folders


def _os_path_cpython(path_str):
    return path_str


def _os_path_ironpython(path_str):
    os_path_obj = openstudio.OpenStudioUtilitiesCore.toPath(path_str)
    return openstudio.Path(os_path_obj)


def _os_vector_len_cpython(vector):
    return len(vector)


def _os_vector_len_ironpython(vector):
    return vector.Count


def _os_create_vector_cpython(val_list, vector):
    return val_list


def _os_create_vector_ironpython(val_list, vector):
    for val in val_list:
        vector.Add(val)
    return vector


if sys.version_info >= (3, 0):  # we are in cPython and can import normally
    import openstudio
    openstudio_model = openstudio.model
    os_path = _os_path_cpython
    os_vector_len = _os_vector_len_cpython
    os_create_vector = _os_create_vector_cpython
else:  # we are in IronPython and we must import the .NET bindings
    try:  # first see if OpenStudio has already been loaded
        import OpenStudio as openstudio
    except ImportError:
        try:
            import clr
        except ImportError as e:  # No .NET being used
            raise ImportError(
                'Failed to import CLR. OpenStudio SDK is unavailable.\n{}'.format(e))
        # check to be sure that the OpenStudio CSharp folder has been installed
        assert hbe_folders.openstudio_path is not None, \
            'No OpenStudio installation was found on this machine.'
        assert hbe_folders.openstudio_csharp_path is not None, \
            'No OpenStudio CSharp folder was found in the OpenStudio installation ' \
            'at:\n{}'.format(os.path.dirname(hbe_folders.openstudio_path))
        # add the OpenStudio DLL to the Common Language Runtime (CLR)
        os_dll = os.path.join(hbe_folders.openstudio_csharp_path, 'OpenStudio.dll')
        clr.AddReferenceToFileAndPath(os_dll)
        if hbe_folders.openstudio_csharp_path not in sys.path:
            sys.path.append(hbe_folders.openstudio_csharp_path)
        import OpenStudio as openstudio
    openstudio_model = openstudio
    os_path = _os_path_ironpython
    os_vector_len = _os_vector_len_ironpython
    os_create_vector = _os_create_vector_ironpython

# load all of the classes used by this package
# geometry classes
OSModel = openstudio_model.Model
OSPoint3dVector = openstudio.Point3dVector
OSPoint3d = openstudio.Point3d
OSShadingSurfaceGroup = openstudio_model.ShadingSurfaceGroup
OSShadingSurface = openstudio_model.ShadingSurface
OSSubSurface = openstudio_model.SubSurface
OSSurface = openstudio_model.Surface
OSSpace = openstudio_model.Space
OSThermalZone = openstudio_model.ThermalZone
OSBuildingStory = openstudio_model.BuildingStory
OSSurfacePropertyOtherSideCoefficients = openstudio_model.SurfacePropertyOtherSideCoefficients
# schedule classes
OSScheduleTypeLimits = openstudio_model.ScheduleTypeLimits
OSScheduleRuleset = openstudio_model.ScheduleRuleset
OSScheduleRule = openstudio_model.ScheduleRule
OSScheduleDay = openstudio_model.ScheduleDay
OSScheduleFixedInterval = openstudio_model.ScheduleFixedInterval
OSExternalFile = openstudio_model.ExternalFile
OSScheduleFile = openstudio_model.ScheduleFile
OSTime = openstudio.Time
OSTimeSeries = openstudio.TimeSeries
OSVector = openstudio.Vector
# material classes
OSMasslessOpaqueMaterial = openstudio_model.MasslessOpaqueMaterial
OSStandardOpaqueMaterial = openstudio_model.StandardOpaqueMaterial
OSRoofVegetation = openstudio_model.RoofVegetation
OSSimpleGlazing = openstudio_model.SimpleGlazing
OSStandardGlazing = openstudio_model.StandardGlazing
OSGas = openstudio_model.Gas
OSGasMixture = openstudio_model.GasMixture
OSBlind = openstudio_model.Blind
OSShade = openstudio_model.Shade
OSWindowPropertyFrameAndDivider = openstudio_model.WindowPropertyFrameAndDivider
# constructions classes
OSConstruction = openstudio_model.Construction
OSMaterialVector = openstudio_model.MaterialVector
OSShadingControl = openstudio_model.ShadingControl
OSConstructionAirBoundary = openstudio_model.ConstructionAirBoundary
OSZoneMixing = openstudio_model.ZoneMixing
# construction set classes
OSDefaultConstructionSet = openstudio_model.DefaultConstructionSet
OSDefaultSurfaceConstructions = openstudio_model.DefaultSurfaceConstructions
OSDefaultSubSurfaceConstructions = openstudio_model.DefaultSubSurfaceConstructions
# internal mass classes
OSInternalMassDefinition = openstudio_model.InternalMassDefinition
OSInternalMass = openstudio_model.InternalMass
# loads classes
OSSpaceType = openstudio_model.SpaceType
OSPeopleDefinition = openstudio_model.PeopleDefinition
OSPeople = openstudio_model.People
OSLightsDefinition = openstudio_model.LightsDefinition
OSLights = openstudio_model.Lights
OSElectricEquipmentDefinition = openstudio_model.ElectricEquipmentDefinition
OSElectricEquipment = openstudio_model.ElectricEquipment
OSGasEquipmentDefinition = openstudio_model.GasEquipmentDefinition
OSGasEquipment = openstudio_model.GasEquipment
OSOtherEquipmentDefinition = openstudio_model.OtherEquipmentDefinition
OSOtherEquipment = openstudio_model.OtherEquipment
OSWaterUseEquipmentDefinition = openstudio_model.WaterUseEquipmentDefinition
OSWaterUseEquipment = openstudio_model.WaterUseEquipment
OSWaterUseConnections = openstudio_model.WaterUseConnections
OSSpaceInfiltrationDesignFlowRate = openstudio_model.SpaceInfiltrationDesignFlowRate
OSDesignSpecificationOutdoorAir = openstudio_model.DesignSpecificationOutdoorAir
OSThermostatSetpointDualSetpoint = openstudio_model.ThermostatSetpointDualSetpoint
OSZoneControlHumidistat = openstudio_model.ZoneControlHumidistat
OSDaylightingControl = openstudio_model.DaylightingControl
# ventilative cooling and AFN classes
OSZoneVentilationWindandStackOpenArea = openstudio_model.ZoneVentilationWindandStackOpenArea
OSZoneVentilationDesignFlowRate = openstudio_model.ZoneVentilationDesignFlowRate
OSAirflowNetworkReferenceCrackConditions = openstudio_model.AirflowNetworkReferenceCrackConditions
OSAirflowNetworkCrack = openstudio_model.AirflowNetworkCrack
OSAirflowNetworkSimpleOpening = openstudio_model.AirflowNetworkSimpleOpening
OSAirflowNetworkHorizontalOpening = openstudio_model.AirflowNetworkHorizontalOpening
# shw System classes
OSPlantLoop = openstudio_model.PlantLoop
OSSetpointManagerScheduled = openstudio_model.SetpointManagerScheduled
OSPumpConstantSpeed = openstudio_model.PumpConstantSpeed
OSWaterHeaterMixed = openstudio_model.WaterHeaterMixed
OSCoilWaterHeatingAirToWaterHeatPump = openstudio_model.CoilWaterHeatingAirToWaterHeatPump
OSFanOnOff = openstudio_model.FanOnOff
OSWaterHeaterHeatPump = openstudio_model.WaterHeaterHeatPump
# HVAC classes
OSZoneHVACIdealLoadsAirSystem = openstudio_model.ZoneHVACIdealLoadsAirSystem
# ems classes
OSOutputVariable = openstudio_model.OutputVariable
OSEnergyManagementSystemProgram = openstudio_model.EnergyManagementSystemProgram
OSEnergyManagementSystemProgramCallingManager = \
    openstudio_model.EnergyManagementSystemProgramCallingManager
OSEnergyManagementSystemSensor = openstudio_model.EnergyManagementSystemSensor
OSEnergyManagementSystemActuator = openstudio_model.EnergyManagementSystemActuator
OSEnergyManagementSystemConstructionIndexVariable = \
    openstudio_model.EnergyManagementSystemConstructionIndexVariable
# generator classes
OSGeneratorPVWatts = openstudio_model.GeneratorPVWatts
OSElectricLoadCenterDistribution = openstudio_model.ElectricLoadCenterDistribution
OSElectricLoadCenterInverterPVWatts = openstudio_model.ElectricLoadCenterInverterPVWatts
# simulation classes
OSRunPeriodControlSpecialDays = openstudio_model.RunPeriodControlSpecialDays
OSMonthOfYear = openstudio.MonthOfYear
OSOutputVariable = openstudio_model.OutputVariable
OSDesignDay = openstudio_model.DesignDay
OSEpwFile = openstudio.EpwFile



# coding=utf-8
"""OpenStudio IdealLoadsAirSystem translator."""
from __future__ import division
from honeybee.altnumber import autosize, no_limit

from honeybee_openstudio.openstudio import OSZoneHVACIdealLoadsAirSystem


def ideal_air_system_to_openstudio(hvac, os_model, room=None):
    """Convert Honeybee IdealAirSystem to OpenStudio ZoneHVACIdealLoadsAirSystem.

    Args:
        hvac: A Honeybee-energy IdealAirSystem to be translated to OpenStudio.
        os_model: The OpenStudio Model object to which the IdealAirSystem
            will be added.
        room: An optional Honeybee Room to be used to set various properties
            of the system (including the EnergyPlus name, and humidity control).
    """
    # create openstudio ideal air system object
    os_ideal_air = OSZoneHVACIdealLoadsAirSystem(os_model)
    if room is None:
        os_ideal_air.setName(hvac.identifier)
    else:
        os_ideal_air.setName('{} Ideal Loads Air System'.format(room.identifier))
    if hvac._display_name is not None:
        hvac.setDisplayName(hvac.display_name)
    # assign the dehumidification based on the room
    os_ideal_air.setDehumidificationControlType('None')  # default when no humidistat
    if room is not None:
        setpoint = room.properties.energy.setpoint
        if setpoint.humidifying_schedule is not None:
            os_ideal_air.setDehumidificationControlType('Humidistat')
            os_ideal_air.setHumidificationControlType('Humidistat')
    # assign the economizer type
    os_ideal_air.setOutdoorAirEconomizerType(hvac.economizer_type)
    # set the sensible and latent heat recovery
    if hvac.sensible_heat_recovery != 0:
        os_ideal_air.setSensibleHeatRecoveryEffectiveness(hvac.sensible_heat_recovery)
        os_ideal_air.setHeatRecoveryType('Sensible')
    else:
        os_ideal_air.setSensibleHeatRecoveryEffectiveness(0)
    if hvac.latent_heat_recovery != 0:
        os_ideal_air.setLatentHeatRecoveryEffectiveness(hvac.latent_heat_recovery)
        os_ideal_air.setHeatRecoveryType('Enthalpy')
    else:
        os_ideal_air.setLatentHeatRecoveryEffectiveness(0)
    # assign the demand controlled ventilation
    if hvac.demand_controlled_ventilation:
        os_ideal_air.setDemandControlledVentilationType('OccupancySchedule')
    else:
        os_ideal_air.setDemandControlledVentilationType('None')
    # set the heating and cooling supply air temperature
    os_ideal_air.setMaximumHeatingSupplyAirTemperature(hvac.heating_air_temperature)
    os_ideal_air.setMinimumCoolingSupplyAirTemperature(hvac.cooling_air_temperature)
    # assign limits to the system's heating capacity
    if hvac.heating_limit == no_limit:
        os_ideal_air.setHeatingLimit('NoLimit')
    else:
        os_ideal_air.setHeatingLimit('LimitCapacity')
    if hvac.heating_limit == autosize:
        os_ideal_air.autosizeMaximumSensibleHeatingCapacity()
    else:
        os_ideal_air.setMaximumSensibleHeatingCapacity(hvac.heating_limit)
    # assign limits to the system's cooling capacity
    if hvac.cooling_limit == no_limit:
        os_ideal_air.setCoolingLimit('NoLimit')
    else:
        os_ideal_air.setCoolingLimit('LimitFlowRateAndCapacity')
    if hvac.cooling_limit == autosize:
        os_ideal_air.autosizeMaximumTotalCoolingCapacity()
        os_ideal_air.autosizeMaximumCoolingAirFlowRate()
    else:
        os_ideal_air.setMaximumTotalCoolingCapacity(hvac.cooling_limit)
        os_ideal_air.autosizeMaximumCoolingAirFlowRate()
    # assign heating availability schedule
    if hvac.heating_availability is not None:
        os_schedule = os_model.getScheduleByName(hvac.heating_availability.identifier)
        if os_schedule.is_initialized():
            os_schedule = os_schedule.get()
            os_ideal_air.setHeatingAvailabilitySchedule(os_schedule)
    # assign cooling availability schedule
    if hvac.cooling_availability is not None:
        os_schedule = os_model.getScheduleByName(hvac.cooling_availability.identifier)
        if os_schedule.is_initialized():
            os_schedule = os_schedule.get()
            os_ideal_air.setCoolingAvailabilitySchedule(os_schedule)
    return os_ideal_air



# coding=utf-8
"""OpenStudio SHWSystem translators."""
from __future__ import division

from honeybee_openstudio.load import _create_constant_schedule
from honeybee_openstudio.openstudio import OSPlantLoop, OSSetpointManagerScheduled, \
    OSPumpConstantSpeed, OSWaterHeaterMixed, OSCoilWaterHeatingAirToWaterHeatPump, \
    OSFanOnOff, OSWaterHeaterHeatPump

ELECTRIC_HEATERS = (
    'Electric_WaterHeater', 'HeatPump_WaterHeater', 'Electric_TanklessHeater')
FUEL_HEATERS = ()
PUMP_HEAD = 29891  # default from OpenStudio App (Pa)
PUMP_EFFICIENCY = 0.9  # default from OpenStudio App
HEAT_PUMP_DEADBAND = 4  # a default for Heat Pumps taken from the OpenStudio App (dc)
MIN_LOOP_TEMP = 10  # default from OpenStudio App (C)
LOOP_TEMP_DIFFERENCE = 5  # default from OpenStudio App (dC)


def _retrieve_source_zone(room_id, os_model):
    """Get the ThermalZone given the room_id for where a SHWSystem is placed."""
    source_zone_ref = os_model.getThermalZoneByName(room_id)
    if source_zone_ref.is_initialized():
        return source_zone_ref.get()
    else:  # the id probably points to a Space instead of a ThermalZone
        source_space_ref = os_model.getSpaceByName(room_id)
        if source_space_ref.is_initialized():
            source_space = source_space_ref.get()
            source_zone_ref = source_space.thermalZone()
            if source_zone_ref.is_initialized():
                return source_zone_ref.get()


def shw_system_to_openstudio(shw, os_shw_connections, total_flow, water_temp, os_model):
    """Translate a Honeybee SHWSystem to OpenStudio PlantLoop with equipment.

    Args:
        shw: The Honeybee-energy SHWSystem object to be translated to OpenStudio.
            This input can also be None, in which case a default system will be
            created using a plant loop with a generic District Heating water heater.
        os_shw_connections: A list of OpenStudio WaterUseConnections objects for
            all of the connections to be made to the system. These are typically
            obtained by using the hot_water_to_openstudio function in this package
            on the Room.properties.energy.service_hot_water object.
        total_flow: A number for the total flow rate of water in the system in m3/s.
            This is typically obtained by summing the individual Room flow rates
            across the system.
        water_temp: A number for the temperature of the water in Celsius. This is
            typically obtained by taking the maximum hot water temperature across
            the individual Room target_temperature.
        os_model: The OpenStudio Model to which the service hot water system
            will be added.
    """
    # set the overall properties if the system is None (default)
    if shw is None:
        shw_id = equip_type = 'Default_District_SHW'
    else:
        shw_id = shw.identifier
        equip_type = shw.equipment_type

    # create the plant loop
    hot_water_plant = OSPlantLoop(os_model)
    hot_water_plant.setName('SHW Loop {}'.format(shw_id))
    target_temp = round(water_temp, 3)
    hot_water_plant.setMaximumLoopTemperature(target_temp)
    hot_water_plant.setMinimumLoopTemperature(MIN_LOOP_TEMP)
    # edit the sizing information to be for a hot water loop
    loop_sizing = hot_water_plant.sizingPlant()
    loop_sizing.setLoopType('Heating')
    loop_sizing.setDesignLoopExitTemperature(target_temp)  
    loop_sizing.setLoopDesignTemperatureDifference(LOOP_TEMP_DIFFERENCE)
    # add a setpoint manager for the loop
    hot_sch_name = '{}C Hot Water'.format(target_temp)
    hot_sch = _create_constant_schedule(hot_sch_name, target_temp, os_model)
    sp_manager = OSSetpointManagerScheduled(os_model, hot_sch)
    sp_manager.addToNode(hot_water_plant.supplyOutletNode())
    # add a constant speed pump for the loop
    hot_water_pump = OSPumpConstantSpeed(os_model)
    hot_water_pump.setName('SHW Pump')
    hot_water_pump.setRatedPumpHead(PUMP_HEAD)
    hot_water_pump.setMotorEfficiency(PUMP_EFFICIENCY)
    hot_water_pump.addToNode(hot_water_plant.supplyInletNode())

    # add the equipment to the plant loop depending on the equipment type
    if equip_type == 'Default_District_SHW':
        # add a district heating system to supply the heat for the loop
        district_hw = OSWaterHeaterMixed(os_model)
        district_hw.setName('Ideal Service Hot Water Heater')
        district_hw.setHeaterFuelType('DistrictHeating')
        district_hw.setOffCycleParasiticFuelType('DistrictHeating')
        district_hw.setOnCycleParasiticFuelType('DistrictHeating')
        district_hw.setHeaterThermalEfficiency(1.0)
        district_hw.setHeaterMaximumCapacity(1000000)
        district_hw.setTankVolume(0)
        district_hw.setHeaterControlType('Modulate')
        a_sch_id = '22C Ambient Condition'
        a_sch = _create_constant_schedule(a_sch_id, 22, os_model)
        district_hw.setAmbientTemperatureSchedule(a_sch)
        district_hw.setOffCycleLossCoefficienttoAmbientTemperature(0)
        district_hw.setOnCycleLossCoefficienttoAmbientTemperature(0)
        hot_water_plant.addSupplyBranchForComponent(district_hw)
        # try to minimize the impact of the pump as much as possible
        hot_water_pump.setEndUseSubcategory('Water Systems')
        hot_water_pump.setMotorEfficiency(0.9)
    else:
        # add a water heater to supply the heat for the loop
        heater = OSWaterHeaterMixed(os_model)
        if equip_type in ELECTRIC_HEATERS:
            heater.setHeaterFuelType('Electricity')
            heater.setOffCycleParasiticFuelType('Electricity')
            heater.setOnCycleParasiticFuelType('Electricity')
        # set the water heater efficiency
        if equip_type == 'HeatPump_WaterHeater':
            heater.setHeaterThermalEfficiency(1.0)
        else:
            heater.setHeaterThermalEfficiency(shw.heater_efficiency)
        # set the ambient condition of the water tank
        if isinstance(shw.ambient_condition, str):  # id of a room where heater is
            heater_in_zone = True
            source_zone = _retrieve_source_zone(shw.ambient_condition, os_model)
            if source_zone is not None:
                heater.setAmbientTemperatureThermalZone(source_zone)
            heater.setAmbientTemperatureIndicator('ThermalZone')
        else:  # a temperature condition in which the heater exists
            heater_in_zone = False
            a_sch_id = '{}C Ambient Condition'.format(shw.ambient_condition)
            a_sch = _create_constant_schedule(a_sch_id, shw.ambient_condition, os_model)
            heater.setAmbientTemperatureSchedule(a_sch)
        # set the ambient loss coefficient
        if heater_in_zone:
            heater.setOffCycleLossFractiontoThermalZone(shw.ambient_loss_coefficient)
            heater.setOnCycleLossFractiontoThermalZone(shw.ambient_loss_coefficient)
        else:
            heater.setOffCycleLossCoefficienttoAmbientTemperature(
                shw.ambient_loss_coefficient)
            heater.setOnCycleLossCoefficienttoAmbientTemperature(
                shw.ambient_loss_coefficient)
        # set the capacity and and controls of the water heater
        heater.setHeaterMaximumCapacity(1000000)
        if equip_type in ('Gas_TanklessHeater', 'Electric_TanklessHeater'):
            heater.setName('SHW Tankless WaterHeater')
            heater.setTankVolume(0)
            heater.setHeaterControlType('Modulate')
            heater.setOffCycleLossCoefficienttoAmbientTemperature(0)
            heater.setOnCycleLossCoefficienttoAmbientTemperature(0)
        else:
            heater.setName('SHW WaterHeater')
            heater.setTankVolume(total_flow)

        # if it's a heat pump system, then add the pump
        if equip_type == 'HeatPump_WaterHeater':
            # create a coil for the heat pump
            heat_pump = OSCoilWaterHeatingAirToWaterHeatPump(os_model)
            heat_pump.setName('SHW HPWH DX Coil')
            heat_pump.setRatedCOP(shw.heater_efficiency)
            # add a fan for the heat pump system
            fan = OSFanOnOff(os_model)
            fan.setName('HPWH Fan')
            fan.setEndUseSubcategory('Water Systems')
            set_p_sch_id = 'HPWH Setpoint - {}'.format(shw_id)
            set_p_sch_val = water_temp + (HEAT_PUMP_DEADBAND * 2)
            setpt_sch = _create_constant_schedule(set_p_sch_id, set_p_sch_val, os_model)
            inlet_sch_id = 'Inlet Air Mixer Fraction - {}'.format(shw_id)
            inlet_sch = _create_constant_schedule(inlet_sch_id, 0.2, os_model)
            # add a water heater to supply the heat for the loop
            heat_sys = OSWaterHeaterHeatPump(os_model, heat_pump, heater, fan, setpt_sch, inlet_sch)
            heat_sys.setDeadBandTemperatureDifference(HEAT_PUMP_DEADBAND)
            source_zone = _retrieve_source_zone(shw.ambient_condition, os_model)
            if source_zone is not None:
                heat_sys.addToThermalZone(source_zone)
            heat_sys.setName('SHW WaterHeater HeatPump')

        # add the water heater to the loop
        hot_water_plant.addSupplyBranchForComponent(heater)

    # add all of the water use connections to the loop
    for shw_conn in os_shw_connections:
        hot_water_plant.addDemandBranchForComponent(shw_conn)
    return hot_water_plant



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

with open('standards-requirements.txt') as f:
    standards_requirements = f.read().splitlines()

setuptools.setup(
    name="honeybee-openstudio",
    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    author="Ladybug Tools",
    author_email="info@ladybug.tools",
    description="Honeybee extension for translating HBJSON files to INP files for eQuest",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ladybug-tools/honeybee-openstudio",
    packages=setuptools.find_packages(exclude=["tests*", "equest_docs*"]),
    install_requires=requirements,
    extras_require={
        'standards': standards_requirements
    },
    include_package_data=True,
    entry_points={
        "console_scripts": ["honeybee-openstudio = honeybee_openstudio.cli:openstudio"]
    },
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: OS Independent"
    ],
    license="AGPL-3.0"
)
